----------------------------------------------------------------------------------------------------
-- FinishState
----------------------------------------------------------------------------------------------------
-- Purpose: Finish state for the state machine.
--
-- Copyright (c) Wopster, 2020
----------------------------------------------------------------------------------------------------

---@class FinishState
---@field public id number
FinishState = {}

local FinishState_mt = Class(FinishState, AbstractState)

---Creates a new finish state.
---@param id number
---@param stateMachine FSM
---@param custom_mt table
---@return FinishState
function FinishState:new(id, stateMachine, isServer, superCup, mission, vehicleList, trafficLight, hud, custom_mt)
    local self = AbstractState:new(id, stateMachine, custom_mt or FinishState_mt)

    self.isServer = isServer
    self.superCup = superCup
    self.mission = mission
    self.vehicleList = vehicleList
    self.trafficLight = trafficLight
    self.hud = hud

    return self
end

---Called on delete.
function FinishState:delete()
    FinishState:superClass().delete(self)
end

---Called on enter.
function FinishState:onEnter(playerUserId, elapsedTime, penalty)
    self.trafficLight:reset()

    if self.isServer then
        for _, vehicle in ipairs(self.vehicleList) do
            if vehicle ~= nil and not vehicle.isDeleted then
                self.mission:removeVehicle(vehicle, false)
                vehicle:delete()
            end
        end
    end

    self.vehicleList = {}

    if elapsedTime ~= nil and penalty ~= nil then
        local user = self.mission.userManager:getUserByUserId(playerUserId)
        if user ~= nil then
            local score = { playerNickname = user:getNickname(), time = SuperCupUtil.millisecondsToClockString(elapsedTime), penalties = penalty, playerId = user:getId(), uuid = user:getUniqueUserId() }

            if not self.isServer then
                g_gui:showInfoDialog({
                    dialogType = DialogElement.TYPE_INFO,
                    text = g_i18n:getText("info_score"):format(score.playerNickname, score.time, score.penalties)
                })
            end

            self.superCup:saveResult(score)
        end
    end

    self.hud:setIsActive(false)

    -- Call at the end to correctly trigger the action listeners.
    FinishState:superClass().onEnter(self)
end
